@@ -50,6 +50,7 @@ both these contexts.
 package XML::XPathScript::Processor;
 
 use strict;
+use warnings;
 
 use Exporter;
 use vars '@ISA', '@EXPORT';
@@ -77,6 +78,8 @@ use vars '@ISA', '@EXPORT';
 		DO_TEXT_AS_CHILD
         );
 
+my $VERSION = 0.1;
+
 =pod "
 
 =over 4
@@ -145,7 +148,13 @@ values considered harmful>).
 =cut "
 
 sub findnodes {
-	$XML::XPathScript::xp->findnodes(@_)
+	if ($XML::XPathScript::xp->isa("XML::XPath")) {
+		return $XML::XPathScript::xp->findnodes(@_);
+	}
+
+	my ($path, $context) = @_;
+	$context = $XML::XPathScript::xp if (!defined $context);
+	return $context->findnodes($path);
 }
 
 =pod "
@@ -162,7 +171,13 @@ L</xpath_to_string>.
 =cut "
 
 sub findvalue {
-	return xpath_to_string(scalar $XML::XPathScript::xp->findvalue(@_));
+	if ($XML::XPathScript::xp->isa("XML::XPath")) {
+		return xpath_to_string(scalar $XML::XPathScript::xp->findvalue(@_));
+	}
+
+	my ($path, $context) = @_;
+	$context = $XML::XPathScript::xp if (!defined $context);
+	return xpath_to_string($context->findvalue($path));
 }
 
 =pod "
@@ -337,7 +352,8 @@ sub call_template {
 }
 
 sub _apply_templates {
-	return join( '', map { translate_node($_) or "" } @_ );
+	no warnings 'uninitialized';
+	return join '', map translate_node($_), @_;
 }
 
 =item  is_element_node ( $object )
@@ -566,10 +582,8 @@ sub translate_text_node {
 		$middle = $node->toString if $retval == DO_TEXT_AS_CHILD;
 	}
 
-	# not pretty, but keep warning mollified
-	return ($trans->{pre} ||''). 
-		   ($middle       ||'').
-		   ($trans->{post}||'');
+	no warnings 'uninitialized';
+	return $trans->{pre} . $middle . $trans->{post};
 }
 
 sub translate_element_node {
@@ -589,6 +603,7 @@ sub translate_element_node {
 		unless( $trans = $translations->{$node_name} )
 		{
 			# no specific and no generic? Okay, okay, return as is...
+			no warnings qw/ uninitialized /;
 			return start_tag($node) . 
                 	_apply_templates( ( $XML::XPathScript::XML_parser eq 'XML::LibXML' ) ? 
 										$node->childNodes : $node->getChildNodes) .
@@ -632,13 +647,15 @@ sub translate_element_node {
 	my $has_kids = $XML::XPathScript::XML_parser eq 'XML::LibXML' ? 
 						$node->hasChildNodes() : $node->getFirstChild();
 	
+	no warnings 'uninitialized';
     my $pre = interpolate($node, $trans->{pre});
 	$pre .= start_tag( $node ) if $trans->{showtag};
+	$pre .= $trans->{intro};
 	$pre .= interpolate($node, $trans->{prechildren}) if $has_kids;
 	
-
-    my $post = '';
+	my $post;
 	$post .= interpolate($node, $trans->{postchildren}) if $has_kids;
+	$post .= $trans->{extro};
 	$post .= end_tag( $node ) if  $trans->{showtag};
 	$post .= interpolate($node, $trans->{post});
 
@@ -656,7 +673,8 @@ sub translate_element_node {
 				if is_element_node( $kid );
         }
         
-		return ($pre||'') . ($middle||'') . ($post||'');
+		no warnings 'uninitialized';
+		return $pre . $middle . $post;
     }
 	
     if($search) 
@@ -703,9 +721,7 @@ sub translate_comment_node {
 		$middle = '' if $retval == DO_SELF_ONLY;
 	}
 	
-	$trans->{pre} ||= '';
-	$trans->{post} ||= '';
-
+	no warnings 'uninitialized';
 	return $trans->{pre}. $middle. $trans->{post};
 }
 
@@ -725,16 +741,17 @@ sub start_tag {
     for my $attr ( ( $XML::XPathScript::XML_parser eq 'XML::LibXML' ) ? 
 						$node->attributes : $node->getAttributeNodes) 
 	{
-	   
+	  
+	  	
 		if( $XML::XPathScript::XML_parser eq 'XML::XPath' )
 	   	{
 	   		$string .= $attr->toString;
 	   	}
 	   	else
 	   	{
-		   my $value = $attr->value;
-		   $value =~ s/'/&quot;/g;
-			$string .= ' ' . $attr->name . "='$value' ";
+			#my $att = $attr->toString( 0, 1 );
+		    	#$att =~ s/'/&quot;/g;
+			$string .= $attr->toString( 0, 1 );
 		}
     }
 
@@ -752,7 +769,7 @@ sub end_tag {
 
 sub interpolate {
     my ($node, $string) = @_;
-
+	
 	# if string is empty or no interpolation,
 	# we return
     return( $string || '' ) unless 
@@ -761,7 +778,9 @@ sub interpolate {
 
 	my $regex = $XML::XPathScript::current->{interpolation_regex};
 	$string =~ s/$regex/ $node->findvalue($1) /egs;
-    return $string || '';
+	
+	no warnings 'uninitialized';
+    return $string;
 }
 
 =back
